% check that the Lyapunov function satisfies all constraints
% by computing all values of mu and n on a grid
% note that this is not a formal proof (see other scripts for this)

clear all

% grid of values for n and mu
ns = round(2.^[1:.1:40]);
mus = [   2.^[-40:.1:-.1] ];
L = 1;
mus = mus * L;
[n,mu] = meshgrid(ns,mus);

% parameters of the Lyapunov function and other constants
C3 = 1/32./n;
alpha = 1/16/L;
delta = min(1/8./n,mu/L/16);
d = alpha./n;
gamma = 1;

h  = (1/2 - 1./n );
c = 4*L./n;
b = -1/4* ( 1./n - 1./n./n) ;
a1 = (1 - 1./n/2)./n/32/L;
a2 = (1 - 1./n/2)./n/16/L;


% compute all parameters
B0 = 2 * delta .* h;
B1 = 2 * ( b - alpha./n .* c);
B2 = 2 * ( alpha./n - d).*h;
B3 = - ( (1-2./n).*a2 + 1./n.*(a1+a2 -2*alpha./n.*b + alpha.*alpha./n./n.*c )-(1-delta).*a2 + L./n.*h.*( alpha./n - d).*( alpha./n - d));
% B4 = - n .* ( (1-2./n).*(a1-2*alpha./n.*b+alpha.*alpha./n./n.*c) - (1-delta).*a1 + L*(1-2./n).*h.*( alpha./n - d).*( alpha./n - d) - h.*(1-delta).*mu.*d.*d ) + B3;
% replace B4 by a stricter value
B4 = - n .* ( (1-2./n).*(a1-2*alpha./n.*b+alpha.*alpha./n./n.*c) - (1-delta).*a1 + L*(1-2./n).*h.*( alpha./n - d).*( alpha./n - d)  ) + B3;
B5 = 2* (  ( delta-1./n).*b - alpha./n.*(1-1./n).*c);
B6 = -2./n .* ( L * h.*(d-alpha./n).*(d-alpha./n) + a1 - 2*alpha./n.*b + alpha.*alpha./n./n.*c );
B7 = 2 * ( L *h.* (d-alpha./n).*(d-alpha./n) + a1 - 2*alpha./n.*b + alpha.*alpha./n./n.*c ) + 2 * (h.* (d-alpha./n).*(1-1./n) - h.*(1-delta).*d );
B8 = 1./n .* ( a1+a2- 2*alpha./n.*b + alpha.*alpha./n./n.*c ) + L./n .* h.* ( alpha./n - d).*( alpha./n - d);
B9 = c .* delta;

C0 = - B0.*mu/2 + B9 +n/4.*B5.*B5 ./ ( B4  ) ;
C1 = B0+B1 + n*L/4.*B6.*B6./( B3 )+n/2.*B5.*(B6+B7)./( B4  )+n*L.*B8;
C2 = -B2-n/4.*B6.*B6./( B3  ) + n/4.*(B6+B7).*(B6+B7)./( B4  );
C13 = C1 + C3;


% now check all constraints
cc = max( -B3, -B4);
cc = max( cc, C2 );
cc = max( cc, C13 );
cc = max( cc, C0+C13.*mu+C2.*mu.*mu   );
cc = max( cc, -  ( L/2.*(2*h-gamma) + c - (d*L+b).*(d*L+b).*n./(n.*a1+a2+n.*mu.*d.*d) ) );
cc = max( cc, - (n.*a1+a2+n.*mu.*d.*d) );
cc = max( cc, 2*h-gamma);
cc = max( cc, - h);

% check that all constraints are indeed less than zero (the following
% number should not be positive (in fact it is equal to zero because of the
% case n=2
max(cc(:))
max(max(cc(:,find(ns>2))))
