% check that the Lyapunov function satisfies all constraints for large
% values of mu (larger than 2/n), with worst case mu=2/n

% note that this leads to a formal proof

clear all

syms n mu delta d  h c b a1 a2 x
syms B0 B1 B2 B3 B4 B5 B6 B7 B8 B9
syms C0_B4 C13_B3B4 C2_B3B4 C3
syms tocheck_1 tocheck_2 tocheck_3 tocheck_4 tocheck_5 tocheck_6 tocheck_7

% parameters of the Lyapunov functions and others
alpha = 1/16;
n = 1/x;
mu = 2/n;
delta = 1/8/n;
d = alpha/n;
gamma = 1;
h = 1/2 - 1/n ;
c = 4/n;
b = -1/4* ( 1/n - 1/n/n) ;
a1 = (1 - 1/n/2)/n/32;
a2 = (1 - 1/n/2)/n/16;
C3 = 1/32/n;

% compute all parameters
B0 = 2 * delta * h;
B1 = 2 * ( b - alpha/n * c);
B2 = 2 * ( alpha/n - d)*h;
B3 = - ( (1-2/n)*a2 + 1/n*(a1+a2 -2*alpha/n*b + alpha*alpha/n/n*c )-(1-delta)*a2);
% modified value of B4 below
B4 = - n * ( (1-2/n)*(a1-2*alpha/n*b+alpha*alpha/n/n*c) - (1-delta)*a1 ) + B3;
B5 = 2* (  ( delta-1/n)*b - alpha/n*(1-1/n)*c);
B6 = -2/n * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c );
B7 = 2 * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c ) + 2 * ( - h*(1-delta)*d );
B8 = 1/n * ( a1+a2- 2*alpha/n*b + alpha*alpha/n/n*c );
B9 = c * delta;

C0_B4 = - B0*mu/2*B4 + B9*B4 +n/4*B5*B5   ;
C13_B3B4 = B0*B3*B4+B1*B3*B4 + n/4*B6*B6*B4+n/2*B5*(B6+B7)*B3+n*B8*B3*B4 + C3*B3*B4;
C2_B3B4 = -B2*B3*B4-n/4*B6*B6*B4 + n/4*(B6+B7)*(B6+B7)*B3;

% all of these must be nonnegative polynomial on [0,1/2], i.e.,
% (a) their value at 1/4 is nonnegative
% (b) there is no root in the interval (0,1/2)
tocheck{1} = simplify(B3);
tocheck{2} = simplify(B4);
tocheck{3} = simplify(-C2_B3B4);
tocheck{4} = simplify(-C13_B3B4);
tocheck{5} = simplify(- ( C0_B4*B3+C13_B3B4*mu+C2_B3B4*mu*mu ));
tocheck{6} = simplify(n*a1+a2+n*mu*d*d);
tocheck{7} = simplify( ( 1/2*(2*h-gamma) + c) * (n*a1+a2+n*mu*d*d) - (d+b)*(d+b)*n );

% performs the checks (should all be one)
for i=1:7
    p = sym2poly(tocheck{i});
    r = roots(p);
    checks(i) = polyval(p,1/4)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=.5))
end

% plot everything
for i=1:7
    latex(tocheck{i})
end



