% check that the Lyapunov function satisfies all constraints for small
% values of mu (smaller than 2/n)

% note that this leads to a formal proof


clear all

syms n mu delta d  h c b a1 a2 x y
syms B0 B1 B2 B3 B4 B5 B6 B7 B8 B9
syms C0_B4 C13_B3B4 C2_B3B4
syms tocheck_1 tocheck_2 tocheck_3 tocheck_4 tocheck_5 tocheck_6 tocheck_7
syms temp


% STEP 1

% parameters of the Lyapunov functions and others
alpha = 1/16;
n = 1/x;
mu = 2/n*y;
delta = mu/16;
d = alpha/n;
gamma = 1;

h = 1/2 - 1/n ;
c = 4/n;
b = -1/4* ( 1/n - 1/n/n) ;
a1 = (1 - 1/n/2)/n/32;
a2 = (1 - 1/n/2)/n/16;


B3 = - ( (1-2/n)*a2 + 1/n*(a1+a2 -2*alpha/n*b + alpha*alpha/n/n*c )-(1-delta)*a2);
B4 = - n * ( (1-2/n)*(a1-2*alpha/n*b+alpha*alpha/n/n*c) - (1-delta)*a1 ) + B3;
B5 = 2* (  ( delta-1/n)*b - alpha/n*(1-1/n)*c);
B6 = -2/n * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c );
B7 = 2 * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c ) + 2 * ( - h*(1-delta)*d );

% B3 decreasing in y
temp = subs(B3,y,1) - subs(B3,y,0); % has to be negative
p = sym2poly(temp);
r = roots(p);
checks(1) = polyval(p,1/2)<0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=.5))

% B3 is positive: check at y=1
temp = subs(B3,y,1) ;
p = sym2poly(temp);
r = roots(p);
checks(2) = polyval(p,1/2)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=.5))

% B4 decreasing in y
temp = subs(B3,y,1) - subs(B3,y,0); % has to be negative
p = sym2poly(temp);
r = roots(p);
checks(3) = polyval(p,1/2)<0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=.5))

% B4 is positive: check at y=1
temp = subs(B4,y,1) ;
p = sym2poly(temp);
r = roots(p);
checks(4) = polyval(p,1/2)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=.5))

% B6+B7 is increasing in y
temp = simplify(subs(B6+B7,y,1) - subs(B6+B7,y,0)); % has to be positive
p = sym2poly(temp);
r = roots(p);
checks(5) = polyval(p,1/4)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=.5))

% B6+B7 is nonnegative: check at y=0
temp =  subs(B6+B7,y,0);
p = sym2poly(temp);
r = roots(p);
checks(6) = polyval(p,1/2)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=.5))


% plots
latex(simplify(B3))
latex(simplify(B4))
latex(simplify(B6+B7))
latex(simplify(B5))



% STEP 2

clear all

syms n mu deltamax d  h c b a1 a2 x y delta
syms B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 B7max
syms C0_B4_mu C13_B3B4 C2_B3B4 C3
syms temp final_check p q

% parameters of the Lyapunov functions and others
alpha = 1/16;
n = 1/x;
mu = 2/n*y;
delta = mu/16;
deltamax = 1/8/n;
d = alpha/n;
gamma = 1;
C3 = 1/32/n;

h = 1/2 - 1/n ;
c = 4/n;
b = -1/4* ( 1/n - 1/n/n) ;
a1 = (1 - 1/n/2)/n/32;
a2 = (1 - 1/n/2)/n/16;


B0 = 2 * delta * h;
B1 = 2 * ( b - alpha/n * c);
B2 = 2 * ( alpha/n - d)*h;
B3 = - ( (1-2/n)*a2 + 1/n*(a1+a2 -2*alpha/n*b + alpha*alpha/n/n*c )-(1-deltamax)*a2);
B4 = - n * ( (1-2/n)*(a1-2*alpha/n*b+alpha*alpha/n/n*c) - (1-deltamax)*a1 ) + B3;
B5 = 2* (  ( delta-1/n)*b - alpha/n*(1-1/n)*c);
B6 = -2/n * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c );
B7 = 2 * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c ) + 2 * ( - h*(1-delta)*d );
B7max = 2 * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c ) + 2 * ( - h*(1-deltamax)*d );
B8 = 1/n * ( a1+a2- 2*alpha/n*b + alpha*alpha/n/n*c );
B9 = c * delta;

C0_B4_mu = - B0/2*B4 + B9*B4/mu +n/4*B5*B5/mu   ;
C13_B3B4 = B0*B3*B4+B1*B3*B4 + n/4*B6*B6*B4+n/2*B5*(B6+B7)*B3+n*B8*B3*B4 + B3*B4*C3;
C2_B3B4 = -B2*B3*B4-n/4*B6*B6*B4 + n/4*(B6+B7)*(B6+B7max)*B3;


% all of these must be nonnegative for y in [0,1] and x in [0,1/5],
% corresponding to n>=5
tocheck_3 = simplify( ( 1/2*(2*h-gamma) + c) * (n*a1+a2+n*mu*d*d) - (d+b)*(d+b)*n );
tocheck_4 = simplify(-C13_B3B4);
tocheck_5 = simplify(-C2_B3B4);
tocheck_6 = simplify(- ( C0_B4_mu*B3+C13_B3B4+C2_B3B4*mu ));
tocheck_7 = simplify(n*a1+a2+n*mu*d*d);



p = sym2poly( subs( tocheck_3 , y , 0 ));
r = roots(p);
checks(1) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/2))

p = sym2poly( subs( tocheck_3 , y , 1 ));
r = roots(p);
checks(2) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/2))

latex(tocheck_3)


p = sym2poly( subs( tocheck_5 , y , 0 ));
r = roots(p);
checks(3) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/2))

p = sym2poly( subs( tocheck_5 , y , 1 ));
r = roots(p);
checks(4) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/2))

latex(tocheck_5)


p = sym2poly( subs( tocheck_7 , y , 0 ));
r = roots(p);
checks(5) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/2))

p = sym2poly( subs( tocheck_7 , y , 1 ));
r = roots(p);
checks(6) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/2))

latex(tocheck_7)



temp = tocheck_4;
param0 =  subs( temp , y , 0 );
param1 =  subs( diff(temp,y), y , 0 );
param2 =  subs( diff(temp,y,2), y , 0 )/2;

% check that second order is positive
p = sym2poly(param2);
r = roots(p);
checks(7) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/5))


% gradient at 0 is negative
p = sym2poly(param1);
r = roots(p);
checks(8) = polyval(p,1/6)<0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/5))


% gradient at 1 is negative
p = sym2poly(param1 + 2*param2);
r = roots(p);
checks(9) = polyval(p,1/6)<0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/5))

% simply check at 1
p = sym2poly( subs( temp , y , 1 ) );
r = roots(p);
checks(10) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/5))


latex(tocheck_4)




temp = tocheck_6;
param0 =  subs( temp , y , 0 );
param1 =  subs( diff(temp,y), y , 0 );
param2 =  subs( diff(temp,y,2), y , 0 )/2;

% check that second order is positive
p = sym2poly(param2);
r = roots(p);
checks(11) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/5))


% check that first order is negative
p = sym2poly(param1);
r = roots(p);
checks(12) = polyval(p,1/6)<0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/5))


% gradient at 1 is negative
p = sym2poly(param1 + 2*param2);
r = roots(p);
checks(13) = polyval(p,1/6)<0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/5))

% simply check at 1
p = sym2poly( subs( temp , y , 1 ) );
r = roots(p);
checks(14) = polyval(p,1/6)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1/5))

latex(tocheck_6)



% STEP 3: n=2

clear all

syms n mu deltamax d  h c b a1 a2 x y delta
syms B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 B7max
syms C0_B4_mu C1_B3B4 C2_B3B4 C3
syms tocheck_1 tocheck_2 tocheck_3 tocheck_4 tocheck_5 tocheck_6 tocheck_7
syms temp final_check p q y


alpha = 1/16;
n = 2;

mu = 2/n*y;
delta = mu/16;
d = alpha/n;
gamma = 1;
C3 = 1/32/n;
h = 1/2 - 1/n ;
c = 4/n;
b = -1/4* ( 1/n - 1/n/n) ;
a1 = (1 - 1/n/2)/n/32;
a2 = (1 - 1/n/2)/n/16;


B0 = 2 * delta * h;
B1 = 2 * ( b - alpha/n * c);
B2 = 2 * ( alpha/n - d)*h;
B3 = - ( (1-2/n)*a2 + 1/n*(a1+a2 -2*alpha/n*b + alpha*alpha/n/n*c )-(1-delta)*a2);
B4 = - n * ( (1-2/n)*(a1-2*alpha/n*b+alpha*alpha/n/n*c) - (1-delta)*a1 ) + B3;
B5 = 2* (  ( delta-1/n)*b - alpha/n*(1-1/n)*c);
B6 = -2/n * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c );
B7 = 2 * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c ) + 2 * ( - h*(1-delta)*d );
B8 = 1/n * ( a1+a2- 2*alpha/n*b + alpha*alpha/n/n*c );
B9 = c * delta;

C0_B4 = - B0*mu/2*B4 + B9*B4 +n/4*B5*B5   ;
C13_B3B4 = B0*B3*B4+B1*B3*B4 + n/4*B6*B6*B4+n/2*B5*(B6+B7)*B3+n*B8*B3*B4 + B3*B4*C3;
C2_B3B4 = -B2*B3*B4-n/4*B6*B6*B4 + n/4*(B6+B7)*(B6+B7)*B3;


% all of these must be nonnegative polynomial on [0,1/2], i.e.,
% (a) their value at 1/4 is nonnegative
% (b) there is no root in the interval (0,1/2)
tocheck_4 = simplify(-C13_B3B4);
tocheck_6 = simplify(- ( C0_B4*B3+C13_B3B4*mu+C2_B3B4*mu*mu ));


p = sym2poly(tocheck_4);
r = roots(p);
checks(1) = polyval(p,1/4)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1))

p = sym2poly(tocheck_6);
r = roots(p);
checks(2) = polyval(p,1/4)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1))






% STEP 4: n=3

clear all

syms n mu deltamax d  h c b a1 a2 x y delta
syms B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 B7max
syms C0_B4_mu C13_B3B4 C2_B3B4 C3
syms tocheck_1 tocheck_2 tocheck_3 tocheck_4 tocheck_5 tocheck_6 tocheck_7
syms temp final_check p q y


alpha = 1/16;
n = 3;

mu = 2/n*y;
delta = mu/16;
d = alpha/n;
gamma = 1;

h = 1/2 - 1/n ;
c = 4/n;
b = -1/4* ( 1/n - 1/n/n) ;
a1 = (1 - 1/n/2)/n/32;
a2 = (1 - 1/n/2)/n/16;
C3 = 1/32/n;

B0 = 2 * delta * h;
B1 = 2 * ( b - alpha/n * c);
B2 = 2 * ( alpha/n - d)*h;
B3 = - ( (1-2/n)*a2 + 1/n*(a1+a2 -2*alpha/n*b + alpha*alpha/n/n*c )-(1-delta)*a2);
B4 = - n * ( (1-2/n)*(a1-2*alpha/n*b+alpha*alpha/n/n*c) - (1-delta)*a1 ) + B3;
B5 = 2* (  ( delta-1/n)*b - alpha/n*(1-1/n)*c);
B6 = -2/n * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c );
B7 = 2 * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c ) + 2 * ( - h*(1-delta)*d );
B8 = 1/n * ( a1+a2- 2*alpha/n*b + alpha*alpha/n/n*c );
B9 = c * delta;

C0_B4 = - B0*mu/2*B4 + B9*B4 +n/4*B5*B5   ;
C13_B3B4 = B0*B3*B4+B1*B3*B4 + n/4*B6*B6*B4+n/2*B5*(B6+B7)*B3+n*B8*B3*B4 + B3*B4*C3;
C2_B3B4 = -B2*B3*B4-n/4*B6*B6*B4 + n/4*(B6+B7)*(B6+B7)*B3;


% all of these must be nonnegative polynomial on [0,1/2], i.e.,
% (a) their value at 1/4 is nonnegative
% (b) there is no root in the interval (0,1/2)
tocheck_4 = simplify(-C13_B3B4);
tocheck_6 = simplify(- ( C0_B4*B3+C13_B3B4*mu+C2_B3B4*mu*mu ));


p = sym2poly(tocheck_4);
r = roots(p);
checks(1) = polyval(p,1/4)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1))

p = sym2poly(tocheck_6);
r = roots(p);
checks(2) = polyval(p,1/4)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1))






% STEP 5: n=4

clear all

syms n mu deltamax d  h c b a1 a2 x y delta
syms B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 B7max
syms C0_B4_mu C13_B3B4 C2_B3B4 C3
syms tocheck_1 tocheck_2 tocheck_3 tocheck_4 tocheck_5 tocheck_6 tocheck_7
syms temp final_check p q y


alpha = 1/16;
n = 4;

mu = 2/n*y;
delta = mu/16;
d = alpha/n;
gamma = 1;
C3 = 1/32/n;
h = 1/2 - 1/n ;
c = 4/n;
b = -1/4* ( 1/n - 1/n/n) ;
a1 = (1 - 1/n/2)/n/32;
a2 = (1 - 1/n/2)/n/16;


B0 = 2 * delta * h;
B1 = 2 * ( b - alpha/n * c);
B2 = 2 * ( alpha/n - d)*h;
B3 = - ( (1-2/n)*a2 + 1/n*(a1+a2 -2*alpha/n*b + alpha*alpha/n/n*c )-(1-delta)*a2);
B4 = - n * ( (1-2/n)*(a1-2*alpha/n*b+alpha*alpha/n/n*c) - (1-delta)*a1 ) + B3;
B5 = 2* (  ( delta-1/n)*b - alpha/n*(1-1/n)*c);
B6 = -2/n * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c );
B7 = 2 * ( a1 - 2*alpha/n*b + alpha*alpha/n/n*c ) + 2 * ( - h*(1-delta)*d );
B8 = 1/n * ( a1+a2- 2*alpha/n*b + alpha*alpha/n/n*c );
B9 = c * delta;

C0_B4 = - B0*mu/2*B4 + B9*B4 +n/4*B5*B5   ;
C13_B3B4 = B0*B3*B4+B1*B3*B4 + n/4*B6*B6*B4+n/2*B5*(B6+B7)*B3+n*B8*B3*B4 + B3*B4*C3;
C2_B3B4 = -B2*B3*B4-n/4*B6*B6*B4 + n/4*(B6+B7)*(B6+B7)*B3;


% all of these must be nonnegative polynomial on [0,1/2], i.e.,
% (a) their value at 1/4 is nonnegative
% (b) there is no root in the interval (0,1/2)
tocheck_4 = simplify(-C13_B3B4);
tocheck_6 = simplify(- ( C0_B4*B3+C13_B3B4*mu+C2_B3B4*mu*mu ));


p = sym2poly(tocheck_4);
r = roots(p);
checks(1) = polyval(p,1/4)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1))

p = sym2poly(tocheck_6);
r = roots(p);
checks(2) = polyval(p,1/4)>0 & all( abs(imag(r))>1e-12 | (r<=0)|(r>=1))




